@@ -20,14 +20,13 @@ import os
 import os.path
 import sys
 import time
-import getopt
 import random
 import fcntl
 import fnmatch
 import re
+from optparse import OptionParser
 
 import output
-from urlgrabber.progress import TextMeter
 import shell
 import yum
 import yum.Errors
@@ -43,8 +42,6 @@ import callback
 import urlgrabber
 import urlgrabber.grabber
 
-__version__ = '2.3.1'
-
 class CliError(yum.Errors.YumBaseError):
    def __init__(self, args=None):
         yum.Errors.YumBaseError.__init__(self)
@@ -59,39 +56,23 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         self.localPackages = [] # for local package handling - possibly needs
                                 # to move to the lower level class
 
-    def doRepoSetup(self, nosack=None):
-        """grabs the repomd.xml for each enabled repository and sets up the basics
-           of the repository"""
+    def doRepoSetup(self, thisrepo=None, dosack=1):
+        """grabs the repomd.xml for each enabled repository 
+           and sets up the basics of the repository"""
         
-        if hasattr(self, 'pkgSack'):
+        if hasattr(self, 'pkgSack') and thisrepo is None:
             self.log(7, 'skipping reposetup, pkgsack exists')
             return
             
-        self.log(2, 'Setting up Repos')
-        if len(self.repos.listEnabled()) < 1:
-            self.errorlog(0, 'No Repositories Available to Set Up')
-            sys.exit(1)
-        for repo in self.repos.listEnabled():
-            if repo.repoXML is not None and len(repo.urls) > 0:
-                continue
-            try:
-                repo.cache = self.conf.getConfigOption('cache')
-                repo.baseurlSetup()
-                repo.dirSetup()
-                self.log(3, 'Baseurl(s) for repo: %s' % repo.urls)
-            except yum.Errors.RepoError, e:
-                self.errorlog(0, '%s' % e)
-                sys.exit(1)
-            try:
-                repo.getRepoXML(text=repo)
-            except yum.Errors.RepoError, e:
-                self.errorlog(0, 'Cannot open/read repomd.xml file for repository: %s' % repo)
-                self.errorlog(0, str(e))
-                sys.exit(1)
-        
-        if not nosack: # so we can make the dirs and grab the repomd.xml but not import the md
+        self.log(2, 'Setting up repositories')
+
+        # Call parent class to do the bulk of work 
+        # (this also ensures that reposetup plugin hook is called)
+        yum.YumBase.doRepoSetup(self, thisrepo=thisrepo)
+
+        if dosack: # so we can make the dirs and grab the repomd.xml but not import the md
             self.log(2, 'Reading repository metadata in from local files')
-            self.doSackSetup()
+            self.doSackSetup(thisrepo=thisrepo)
     
         
     def getOptionsConfig(self, args):
@@ -102,62 +83,130 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         # setup our errorlog object 
         self.errorlog = Logger(threshold=2, file_object=sys.stderr)
     
-        # our default config file location
-        yumconffile = None
-        if os.access("/etc/yum.conf", os.R_OK):
-            yumconffile = "/etc/yum.conf"
-    
+        def repo_optcb(optobj, opt, value, parser):
+            '''Callback for the enablerepo and disablerepo option. 
+            
+            Combines the values given for these options while preserving order
+            from command line.
+            '''
+            dest = eval('parser.values.%s' % optobj.dest)
+            dest.append((opt, value))
+
+        self.optparser = YumOptionParser(base=self, usage='''\
+yum [options] < update | install | info | remove | list |
+    clean | provides | search | check-update | groupinstall | 
+    groupupdate | grouplist | groupinfo | groupremove |
+    makecache | localinstall | erase | upgrade | whatprovides |
+    localupdate | resolvedep | shell | deplist >''')
+
+        self.optparser.add_option("-t", "--tolerant", dest="tolerant",
+                action="store_true", default=False, help="be tolerant of errors")
+        self.optparser.add_option("-C", "", dest="cacheonly",
+                action="store_true", default=False, 
+                help="run entirely from cache, don't update cache")
+        self.optparser.add_option("-c", "", dest="conffile", action="store", 
+                default='/etc/yum.conf', help="config file location", 
+                metavar=' [config file]')
+        self.optparser.add_option("-R", "", dest="sleeptime", action="store", 
+                type='int', default=None, help="maximum command wait time",
+                metavar=' [minutes]')
+        self.optparser.add_option("-d", "", dest="debuglevel", action="store", 
+                default=None, help="debugging output level", type='int',
+                metavar=' [debug level]')
+        self.optparser.add_option("-e", "", dest="errorlevel", action="store", 
+                default=None, help="error output level", type='int',
+                metavar=' [error level]')
+        self.optparser.add_option("-y", "", dest="assumeyes",
+                action="store_true", default=False, 
+                help="answer yes for all questions")
+        self.optparser.add_option("", "--version", dest="version",
+                default=False, action="store_true", 
+                help="show Yum version and exit")
+        self.optparser.add_option("", "--installroot", dest="installroot",
+                action="store", default=None, help="set install root", 
+                metavar='[path]')
+        self.optparser.add_option("", "--enablerepo", action='callback',
+                type='string', callback=repo_optcb, dest='repos', default=[],
+                help="enable one or more repositories (wildcards allowed)",
+                metavar='[repo]')
+        self.optparser.add_option("", "--disablerepo", action='callback',
+                type='string', callback=repo_optcb, dest='repos', default=[],
+                help="disable one or more repositories (wildcards allowed)",
+                metavar='[repo]')
+        self.optparser.add_option("", "--exclude", dest="exclude", default=[], 
+                action="append", help="exclude package(s) by name or glob",
+                metavar='[package]')
+        self.optparser.add_option("", "--obsoletes", dest="obsoletes",
+                default=False, action="store_true", 
+                help="enable obsoletes processing during updates")
+        self.optparser.add_option("", "--noplugins", dest="noplugins",
+                default=False, action="store_true", 
+                help="disable Yum plugins")
+        
+        # Parse only command line options that affect basic yum setup
         try:
-            gopts, self.cmds = getopt.getopt(args, 'tCc:hR:e:d:y', ['help',
-                                                            'version',
-                                                            'installroot=',
-                                                            'enablerepo=',
-                                                            'disablerepo=',
-                                                            'exclude=',
-                                                            'obsoletes',
-                                                            'rss-filename=',
-                                                            'tolerant'])
-        except getopt.error, e:
-            self.errorlog(0, _('Options Error: %s') % e)
+            args = _filtercmdline(
+                        ('--noplugins',), 
+                        ('-c', '-d', '-e', '--installroot'), 
+                        args,
+                    )
+        except ValueError:
             self.usage()
             sys.exit(1)
-            
-        # get the early options out of the way
-        # these are ones that:
-        #  - we need to know about and do NOW
-        #  - answering quicker is better
-        #  - give us info for parsing the others
-        
-        # our sleep variable for the random start time
-        sleeptime=0
-        root = '/'
-        installroot = None
-        conffile = '/etc/yum.conf'
-        
+        opts = self.optparser.parse_args(args=args)[0]
+
         try: 
-            for o,a in gopts:
-                if o == '--version':
-                    print __version__
-                    sys.exit(0)
-                if o == '--installroot':
-                    installroot = a
-                if o == '-c':
-                    conffile = a
-            
-            # if the conf file is inside the  installroot - use that.
+            # If the conf file is inside the  installroot - use that.
             # otherwise look for it in the normal root
-            if installroot:
-                if os.access(installroot + '/' + conffile, os.R_OK):
-                    conffile = installroot + '/' + conffile
-                    
-                root = installroot
+            if opts.installroot:
+                if os.access(opts.installroot+'/'+opts.conffile, os.R_OK):
+                    opts.conffile = opts.installroot+'/'+opts.conffile
+                root=opts.installroot
+            else:
+                root = '/'
                     
+            # Parse the configuration file
             try:
-                self.doConfigSetup(fn = conffile, root = root)
+                self.doConfigSetup(fn=opts.conffile, root=root)
             except yum.Errors.ConfigError, e:
                 self.errorlog(0, _('Config Error: %s') % e)
                 sys.exit(1)
                 
+            # Initialise logger object
+            self.log=Logger(threshold=self.conf.getConfigOption('debuglevel'), 
+                    file_object=sys.stdout)
+
+            # Setup debug and error levels
+            if opts.debuglevel is not None:
+                self.log.threshold=opts.debuglevel
+                self.conf.setConfigOption('debuglevel', opts.debuglevel)
+
+            if opts.errorlevel is not None:
+                self.errorlog.threshold=opts.errorlevel
+                self.conf.setConfigOption('errorlevel', opts.errorlevel)
+
+        except ValueError, e:
+            self.errorlog(0, _('Options Error: %s') % e)
+            self.usage()
+            sys.exit(1)
+
+        # Initialise plugins if cmd line and config file say these should be in
+        # use (this may add extra command line options)
+        if not opts.noplugins and self.conf.plugins:
+            self.doPluginSetup(self.optparser)
+
+        # Now parse the command line for real
+        (opts, self.cmds) = self.optparser.parse_args()
+
+        # Just print out the version if that's what the user wanted
+        if opts.version:
+            print yum.__version__
+            sys.exit(0)
+
+        # Let the plugins know what happened on the command line
+        self.plugins.setCmdLine(opts, self.cmds)
+
+        try:
             # config file is parsed and moving us forward
             # set some things in it.
                 
@@ -165,12 +214,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
             self.conf.setConfigOption('uid', os.geteuid())
 
             # version of yum
-            self.conf.setConfigOption('yumversion', __version__)
-            
-            
-            # we'd like to have a log object now
-            self.log=Logger(threshold=self.conf.getConfigOption('debuglevel'), file_object = 
-                                                                        sys.stdout)
+            self.conf.setConfigOption('yumversion', yum.__version__)
             
             # syslog-style log
             if self.conf.getConfigOption('uid') == 0:
@@ -196,79 +240,53 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                 self.filelog = Logger(threshold = 10, file_object = None, 
                                 preprefix = self.printtime)
             
-        
-            # now the rest of the options
-            for o,a in gopts:
-                if o == '-d':
-                    self.log.threshold=int(a)
-                    self.conf.setConfigOption('debuglevel', int(a))
-                elif o == '-e':
-                    self.errorlog.threshold=int(a)
-                    self.conf.setConfigOption('errorlevel', int(a))
-                elif o == '-y':
-                    self.conf.setConfigOption('assumeyes',1)
-                elif o in ['-h', '--help']:
+            # Handle remaining options
+            if opts.assumeyes:
+                self.conf.setConfigOption('assumeyes',1)
+
+            if opts.cacheonly:
+                self.conf.setConfigOption('cache', 1)
+
+            if opts.sleeptime is not None:
+                sleeptime = random.randrange(opts.sleeptime*60)
+            else:
+                sleeptime = 0
+
+            if opts.obsoletes:
+                self.conf.setConfigOption('obsoletes', 1)
+
+            if opts.installroot:
+                self.conf.setConfigOption('installroot', opts.installroot)
+
+            for exclude in opts.exclude:
+                try:
+                    excludelist = self.conf.getConfigOption('exclude')
+                    excludelist.append(exclude)
+                    self.conf.setConfigOption('exclude', excludelist)
+                except yum.Errors.ConfigError, e:
+                    self.errorlog(0, _(e))
                     self.usage()
-                    sys.exit(0)
-                elif o == '-C':
-                    self.conf.setConfigOption('cache', 1)
-                elif o == '-R':
-                    sleeptime = random.randrange(int(a)*60)
-                elif o == '--obsoletes':
-                    self.conf.setConfigOption('obsoletes', 1)
-                elif o == '--installroot':
-                    self.conf.setConfigOption('installroot', a)
-                elif o == '--rss-filename':
-                    self.conf.setConfigOption('rss-filename', a)
-                elif o == '--enablerepo':
-                    try:
-                        self.repos.enableRepo(a)
-                    except yum.Errors.ConfigError, e:
-                        self.errorlog(0, _(e))
-                        self.usage()
-                        sys.exit(1)
-                elif o == '--disablerepo':
-                    try:
-                        self.repos.disableRepo(a)
-                    except yum.Errors.ConfigError, e:
-                        self.errorlog(0, _(e))
-                        self.usage()
-                        sys.exit(1)
-                        
-                elif o == '--exclude':
-                    try:
-                        excludelist = self.conf.getConfigOption('exclude')
-                        excludelist.append(a)
-                        self.conf.setConfigOption('exclude', excludelist)
-                    except yum.Errors.ConfigError, e:
-                        self.errorlog(0, _(e))
-                        self.usage()
-                        sys.exit(1)
-                
+                    sys.exit(1)
+               
+            # Process repo enables and disables in order
+            for opt, repoexp in opts.repos:
+                try:
+                    if opt == '--enablerepo':
+                        self.repos.enableRepo(repoexp)
+                    elif opt == '--disablerepo':
+                        self.repos.disableRepo(repoexp)
+                except yum.Errors.ConfigError, e:
+                    self.errorlog(0, _(e))
+                    self.usage()
+                    sys.exit(1)
                             
         except ValueError, e:
             self.errorlog(0, _('Options Error: %s') % e)
             self.usage()
             sys.exit(1)
-            
-        # if we're below 2 on the debug level we don't need to be outputting
-        # progress bars - this is hacky - I'm open to other options
-        # One of these is a download
-        if self.conf.getConfigOption('debuglevel') < 2 or not sys.stdout.isatty():
-            self.repos.setProgressBar(None)
-            self.repos.callback = None
-        else:
-            self.repos.setProgressBar(TextMeter(fo=sys.stdout))
-            self.repos.callback = output.CacheProgressCallback(self.log,
-                    self.errorlog, self.filelog)
-
-        # setup our failure report for failover
-        freport = (self.failureReport,(),{})
-        self.repos.setFailureCallback(freport)
-        
-        # setup our depsolve progress callback
-        dscb = output.DepSolveProgressCallBack(self.log, self.errorlog)
-        self.dsCallback = dscb
+         
+        # setup the progress bars/callbacks
+        self.setupProgessCallbacks()
         
         # save our original args out
         self.args = args
@@ -278,8 +296,8 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
             self.cmdstring += '%s ' % arg
 
         try:
-            self.parseCommands() # before we exit check over the base command + args
-                             # make sure they match/make sense
+            self.parseCommands() # before we return check over the base command + args
+                                 # make sure they match/make sense
         except CliError:
             sys.exit(1)
     
@@ -319,9 +337,9 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                                 'grouplist', 'groupupdate', 'groupinstall',
                                 'groupremove', 'groupinfo', 'makecache',
                                 'clean', 'remove', 'provides', 'check-update',
-                                'search', 'generate-rss', 'upgrade', 
-                                'whatprovides', 'localinstall', 'localupdate',
-                                'resolvedep', 'shell']:
+                                'search', 'upgrade', 'whatprovides',
+                                'localinstall', 'localupdate',
+                                'resolvedep', 'shell', 'deplist']:
             self.usage()
             raise CliError
     
@@ -357,7 +375,7 @@ For more information contact your distribution or package provider.
                         raise CliError
 
                 
-        if self.basecmd in ['install', 'erase', 'remove', 'localinstall', 'localupdate']:
+        if self.basecmd in ['install', 'erase', 'remove', 'localinstall', 'localupdate', 'deplist']:
             if len(self.extcmds) == 0:
                 self.errorlog(0, _('Error: Need to pass a list of pkgs to %s') % self.basecmd)
                 self.usage()
@@ -384,15 +402,6 @@ For more information contact your distribution or package provider.
                     self.usage()
                     raise CliError
                     
-        elif self.basecmd == 'generate-rss':
-            if len(self.extcmds) == 0:
-                self.extcmds.insert(0, 'recent')
-            
-            if self.extcmds[0] not in ['updates', 'recent']:
-                self.errorlog(0, _("Error: generate-rss takes no argument, 'updates' or 'recent'."))
-                self.usage()
-                raise CliError
-            
         elif self.basecmd == 'shell':
             if len(self.extcmds) == 0:
                 self.log(3, "No argument to shell")
@@ -410,14 +419,14 @@ For more information contact your distribution or package provider.
                 raise CliError
               
         elif self.basecmd in ['list', 'check-update', 'info', 'update', 'upgrade',
-                              'generate-rss', 'grouplist', 'makecache',
-                              'resolvedep']:
+                              'grouplist', 'makecache', 'resolvedep']:
             pass
     
         else:
             self.usage()
             raise CliError
 
+
     def doShell(self):
         """do a shell-like interface for yum commands"""
 
@@ -525,34 +534,13 @@ For more information contact your distribution or package provider.
             else:
                 return result, []
             
-            
-        elif self.basecmd == 'generate-rss':
-            self.log(2, 'Setting up RSS Generation')
-            titles = { 'recent': 'Recent Packages',
-                       'updates': 'Updated Packages'}
-            try:
-                pkgtype = self.extcmds[0]
-                ypl = self.returnPkgLists()
-                this_pkg_list = getattr(ypl, pkgtype)
-                if len(this_pkg_list) > 0:
-                    needrepos = []
-                    for po in this_pkg_list:
-                        if po.repoid not in needrepos:
-                            needrepos.append(po.repoid)
+        elif self.basecmd in ['deplist']:
+           self.log(2, "Finding dependencies: ")
+           try:
+              return self.deplist()
+           except yum.Errors.YumBaseError, e:
+              return 1, [str(e)]
 
-                    self.log(2, 'Importing Changelog Metadata')
-                    self.repos.populateSack(with='otherdata', which=needrepos)
-                    self.log(2, 'Generating RSS File for %s' % pkgtype)
-                        
-                    self.listPkgs(this_pkg_list, titles[pkgtype], outputType='rss')
-                else:
-                    self.errorlog(0, 'No Recent Packages')
-
-            except yum.Errors.YumBaseError, e:
-                return 1, [str(e)]
-            else:
-                return 0, ['Done']
-                
         elif self.basecmd == 'clean':
             self.conf.setConfigOption('cache', 1)
             return self.cleanCli()
@@ -562,7 +550,7 @@ For more information contact your distribution or package provider.
 
             self.log(2, "Setting up Group Process")
 
-            self.doRepoSetup(nosack=1)
+            self.doRepoSetup(dosack=0)
             try:
                 self.doGroupSetup()
             except yum.Errors.GroupsError:
@@ -622,7 +610,7 @@ For more information contact your distribution or package provider.
             self.log(2, "This may take a while depending on the speed of this computer")
             self.log(3, '%s' % self.pickleRecipe())
             try:
-                self.doRepoSetup(nosack=1)
+                self.doRepoSetup(dosack=0)
                 self.repos.populateSack(with='metadata', pickleonly=1)
                 self.repos.populateSack(with='filelists', pickleonly=1)
                 self.repos.populateSack(with='otherdata', pickleonly=1)
@@ -654,12 +642,10 @@ For more information contact your distribution or package provider.
         self.reportDownloadSize(downloadpkgs)
         
         # confirm with user
-        if not self.conf.getConfigOption('assumeyes'):
+        if self._promptWanted():
             if not self.userconfirm():
                 self.log(0, 'Exiting on user Command')
-                return
-
-        
+                return 1
 
         self.log(2, 'Downloading Packages:')
         problems = self.downloadPkgs(downloadpkgs) 
@@ -675,7 +661,7 @@ For more information contact your distribution or package provider.
 
         # Check GPG signatures
         if self.gpgsigcheck(downloadpkgs) != 0:
-            return
+            return 1
         
         self.log(2, 'Running Transaction Test')
         tsConf = {}
@@ -694,8 +680,6 @@ For more information contact your distribution or package provider.
         self.populateTs(keepold=0) # sigh
         tserrors = self.ts.test(testcb, conf=tsConf)
         del testcb
-        # put it back
-        self.dsCallback = dscb
         
         self.log(2, 'Finished Transaction Test')
         if len(tserrors) > 0:
@@ -711,28 +695,24 @@ For more information contact your distribution or package provider.
         self.populateTs(keepold=0) # populate the ts
         self.ts.check() #required for ordering
         self.ts.order() # order
-        
+
+        # put back our depcheck callback
+        self.dsCallback = dscb
+
         output = 1
         if self.conf.debuglevel < 2:
             output = 0
         cb = callback.RPMInstallCallback(output=output)
         cb.filelog = self.filelog # needed for log file output
         cb.tsInfo = self.tsInfo
-        
-        # run ts
+
         self.log(2, 'Running Transaction')
-        
-        errors = self.ts.run(cb.callback, '')
-        if errors:
-            errstring = 'Error in Transaction: '
-            for descr in errors:
-                errstring += '  %s\n' % str(descr)
-            
-            raise yum.Errors.YumBaseError, errstring
+        self.runTransaction(cb=cb)
 
         # close things
         self.log(1, self.postTransactionOutput())
-
+        return 0
+        
     def gpgsigcheck(self, pkgs):
         '''Perform GPG signature verification on the given packages, installing
         keys if possible
@@ -760,55 +740,65 @@ For more information contact your distribution or package provider.
                         'unattended.\nUse "-y" to override.'
 
                 repo = self.repos.getRepo(po.repoid)
-                keyurl = repo.gpgkey
-                self.log(0, 'Retrieving GPG key from %s' % keyurl)
+                keyurls = repo.gpgkey
+                key_installed = False
 
-                # Go get the GPG key from the given URL
-                try:
-                    rawkey = urlgrabber.urlread(keyurl, limit=9999)
-                except urlgrabber.grabber.URLGrabError, e:
-                    raise yum.Errors.YumBaseError(
-                            'GPG key retrieval failed: ' + str(e))
+                for keyurl in keyurls:
+                    self.log(0, 'Retrieving GPG key from %s' % keyurl)
 
-                # Parse the key
-                try:
-                    keyinfo = yum.misc.getgpgkeyinfo(rawkey)
-                    keyid = keyinfo['keyid']
-                    hexkeyid = yum.misc.keyIdToRPMVer(keyid).upper()
-                    timestamp = keyinfo['timestamp']
-                    userid = keyinfo['userid']
-                except ValueError, e:
+                    # Go get the GPG key from the given URL
+                    try:
+                        rawkey = urlgrabber.urlread(keyurl, limit=9999)
+                    except urlgrabber.grabber.URLGrabError, e:
+                        raise yum.Errors.YumBaseError(
+                                'GPG key retrieval failed: ' + str(e))
+
+                    # Parse the key
+                    try:
+                        keyinfo = yum.misc.getgpgkeyinfo(rawkey)
+                        keyid = keyinfo['keyid']
+                        hexkeyid = yum.misc.keyIdToRPMVer(keyid).upper()
+                        timestamp = keyinfo['timestamp']
+                        userid = keyinfo['userid']
+                    except ValueError, e:
+                        raise yum.Errors.YumBaseError, \
+                                'GPG key parsing failed: ' + str(e)
+
+                    # Check if key is already installed
+                    if yum.misc.keyInstalled(self.read_ts, keyid, timestamp) >= 0:
+                        self.log(0, 'GPG key at %s (0x%s) is already installed' % (
+                                keyurl,
+                                hexkeyid
+                                ))
+                        continue
+
+                    # Try installing/updating GPG key
+                    self.log(0, 'Importing GPG key 0x%s "%s"' % (hexkeyid, userid))
+                    if not self.conf.getConfigOption('assumeyes'):
+                        if not self.userconfirm():
+                            self.log(0, 'Exiting on user command')
+                            return 1
+            
+                    # Import the key
+                    result = self.ts.pgpImportPubkey(yum.misc.procgpgkey(rawkey))
+                    if result != 0:
+                        raise yum.Errors.YumBaseError, \
+                                'Key import failed (code %d)' % result
+                    self.log(1, 'Key imported successfully')
+                    key_installed = True
+
+                if not key_installed:
                     raise yum.Errors.YumBaseError, \
-                            'GPG key parsing failed: ' + str(e)
+                        'The GPG keys listed for the "%s" repository are ' \
+                        'already installed but they are not correct for this ' \
+                        'package.\n' \
+                        'Check that the correct key URLs are configured for ' \
+                        'this repository.' % (repo.name)
 
-                # Check if key is already installed
-                if yum.misc.keyInstalled(self.read_ts, keyid, timestamp) >= 0:
-                    self.log(0, '\n')
-                    raise yum.Errors.YumBaseError, \
-                            'The GPG key at %s (0x%s) \n' \
-                            'is already installed but is not the correct '\
-                            'key for this package.\nCheck that this is the ' \
-                            'correct key for the "%s" repository.' % \
-                                (keyurl, hexkeyid, repo.name)
-
-                # Try installing/updating GPG key
-                self.log(0, 'Importing GPG key 0x%s "%s"' % (hexkeyid, userid))
-                if not self.conf.getConfigOption('assumeyes'):
-                    if not self.userconfirm():
-                        self.log(0, 'Exiting on user command')
-                        return 1
-        
-                # Import the key
-                result = self.ts.pgpImportPubkey(yum.misc.procgpgkey(rawkey))
-                if result != 0:
-                    raise yum.Errors.YumBaseError, \
-                            'Key import failed (code %d)' % result
-                self.log(1, 'Key imported successfully')
-    
-                # Check if the key helped
+                # Check if the newly installed keys helped
                 result, errmsg = self.sigCheckPkg(po)
                 if result != 0:
-                    self.log(0, "Key import didn't help, wrong key?")
+                    self.log(0, "Import of key(s) didn't help, wrong key(s)?")
                     raise yum.Errors.YumBaseError, errmsg
 
             else:
@@ -850,7 +840,8 @@ For more information contact your distribution or package provider.
                          # no matter what we don't go looking at repos
 
             arglist = [arg]
-            exactmatch, matched, unmatched = parsePackages(avail, arglist)
+            exactmatch, matched, unmatched = parsePackages(avail, arglist, 
+                                                               casematch=1)
             if len(unmatched) > 0: # if we get back anything in unmatched, check it for a virtual-provide
                 arg = unmatched[0] #only one in there
                 self.log(3, 'Checking for virtual provide or file-provide for %s' % arg)
@@ -867,7 +858,7 @@ For more information contact your distribution or package provider.
                     matched.extend(mtch)
             
             installable = yum.misc.unique(exactmatch + matched)
-            exactarch = self.conf.exactarch
+            exactarchlist = self.conf.exactarchlist
             
             # we look through each returned possibility and rule out the
             # ones that we obviously can't use
@@ -893,7 +884,7 @@ For more information contact your distribution or package provider.
                         (n2, a2, e2, v2, r2) = instTup
                         rc = compareEVR((e2, v2, r2), (pkg.epoch, pkg.version, pkg.release))
                         if rc < 0: # we're newer - this is an update, pass to them
-                            if exactarch:
+                            if n2 in exactarchlist:
                                 if pkg.arch == a2:
                                     passToUpdate.append(pkg.pkgtup)
                             else:
@@ -1003,7 +994,8 @@ For more information contact your distribution or package provider.
                 updatesPo.extend(self.pkgSack.searchNevra(name=n, arch=a, epoch=e, 
                                  ver=v, rel=r))
                                  
-            exactmatch, matched, unmatched = yum.packages.parsePackages(updatesPo, userlist)
+            exactmatch, matched, unmatched = yum.packages.parsePackages(
+                                                updatesPo, userlist, casematch=1)
             for userarg in unmatched:
                 if not quiet:
                     self.errorlog(1, 'Could not find update match for %s' % userarg)
@@ -1042,9 +1034,23 @@ For more information contact your distribution or package provider.
             installed.append(po)
         
         if len(userlist) > 0: # if it ain't well, that'd be real _bad_ :)
-            exactmatch, matched, unmatched = yum.packages.parsePackages(installed, userlist)
+            exactmatch, matched, unmatched = yum.packages.parsePackages(
+                                             installed, userlist, casematch=1)
             erases = yum.misc.unique(matched + exactmatch)
-        
+
+        if unmatched:
+            for arg in unmatched:
+                try:
+                    depmatches = self.returnInstalledPackagesByDep(arg)
+                except yum.Errors.YumBaseError, e:
+                    self.errorlog(0, _('%s') % e)
+                    continue
+                    
+                if not depmatches:
+                    self.errorlog(0, _('No Match for argument: %s') % arg)
+                else:
+                    erases.extend(depmatches)
+            
         for pkg in erases:
             self.tsInfo.addErase(pkg)
         
@@ -1101,7 +1107,7 @@ For more information contact your distribution or package provider.
                 (n2, a2, e2, v2, r2) = installed_pkg.pkgtup
                 rc = compareEVR((e2, v2, r2), (e, v, r))
                 if rc < 0: # we're newer - this is an update, pass to them
-                    if self.conf.exactarch:
+                    if n2 in self.conf.exactarchlist:
                         if a == a2:
                             updatepkgs.append((po, installed_pkg))
                             continue
@@ -1205,6 +1211,18 @@ For more information contact your distribution or package provider.
             return 0, ['No Matches found']
         return 0, []
 
+    def deplist(self, args=None):
+       """cli wrapper method for findDeps method takes a list of packages and 
+       returns a formatted deplist for that package"""
+
+       if not args:
+          args = self.extcmds
+
+       results = self.findDeps(args)
+       self.depListOutput(results)
+
+       return 0, []
+
     def provides(self, args=None):
         """use the provides methods in the rpmdb and pkgsack to produce a list 
            of items matching the provides strings. This is a cli wrapper to the 
@@ -1438,37 +1456,89 @@ For more information contact your distribution or package provider.
             return 0, ['No packages to remove from groups']
 
 
+    def _promptWanted(self):
+        # shortcut for the always-off/always-on options
+        if self.conf.getConfigOption('assumeyes'):
+            return False
+        if self.conf.getConfigOption('alwaysprompt'):
+            return True
+        
+        # prompt if:
+        #  package was added to fill a dependency
+        #  package is being removed
+        #  package wasn't explictly given on the command line
+        for txmbr in self.tsInfo.getMembers():
+            if txmbr.isDep or \
+                   txmbr.ts_state == 'e' or \
+                   txmbr.name not in self.extcmds:
+                return True
+        
+        # otherwise, don't prompt        
+        return False
 
-            
-        
-        
     def usage(self):
-        print _("""
-    Usage:  yum [options] < update | install | info | remove | list |
-            clean | provides | search | check-update | groupinstall | 
-            groupupdate | grouplist | groupinfo | groupremove | generate-rss |
-            makecache | localinstall | shell >
-                
-        Options:
-        -c [config file] - specify the config file to use
-        -e [error level] - set the error logging level
-        -d [debug level] - set the debugging level
-        -y - answer yes to all questions
-        -R [time in minutes] - set the max amount of time to randomly run in
-        -C run from cache only - do not update the cache
-        --installroot=[path] - set the install root (default '/')
-        --version - output the version of yum
-        --rss-filename=[path/filename] - set the filename to generate rss to
-        --exclude=package to exclude
-        --disablerepo=repository id to disable (overrides config file)
-        --enablerepo=repository id to enable (overrides config file)
+        '''Print out command line usage
+        '''
+        print
+        self.optparser.print_help()
 
-        -h, --help  - this screen
-    """)
+class YumOptionParser(OptionParser):
+    '''Subclass that makes some minor tweaks to make OptionParser do things the
+    "yum way".
+    '''
 
+    def __init__(self, base, **kwargs):
+        OptionParser.__init__(self, **kwargs)
+        self.base = base
 
-           
+    def error(self, msg):
+        '''This method is overridden so that error output goes to errorlog
+        '''
+        self.print_usage()
+        self.base.errorlog(0, "Command line error: "+msg)
+        sys.exit(1)
 
+def _filtercmdline(novalopts, valopts, args):
+    '''Keep only specific options from the command line argument list
 
-        
+    This function allows us to peek at specific command line options when using
+    the optparse module. This is useful when some options affect what other
+    options should be available.
 
+    @param novalopts: A sequence of options to keep that don't take an argument.
+    @param valopts: A sequence of options to keep that take a single argument.
+    @param args: The command line arguments to parse (as per sys.argv[:1]
+    @return: A list of strings containing the filtered version of args.
+
+    Will raise ValueError if there was a problem parsing the command line.
+    '''
+    out = []
+    args = list(args)       # Make a copy because this func is destructive
+
+    while len(args) > 0:
+        a = args.pop(0)
+        if '=' in a:
+            opt, _ = a.split('=', 1)
+            if opt in valopts:
+                out.append(a)
+
+        elif a in novalopts:
+            out.append(a)
+
+        elif a in valopts:
+            if len(args) < 1:
+                raise ValueError
+            next = args.pop(0)
+            if next.startswith('-'):
+                raise ValueError
+
+            out.extend([a, next])
+       
+        else:
+            # Check for single letter options that take a value, where the
+            # value is right up against the option
+            for opt in valopts:
+                if len(opt) == 2 and a.startswith(opt):
+                    out.append(a)
+
+    return out
