@@ -22,7 +22,8 @@ import os.path
 import sys
 import time
 from i18n import _
-import libxml2
+
+from urlgrabber.progress import TextMeter
 
 try:
     import readline
@@ -34,9 +35,13 @@ import yum.Errors
 class YumOutput:
 
     def printtime(self):
-        return time.strftime('%b %d %H:%M:%S ', time.localtime(time.time()))
-        
-    
+        months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+                  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
+        now = time.localtime(time.time())
+        ret = months[int(time.strftime('%m', now)) - 1] + \
+              time.strftime(' %d %T ', now)
+        return ret
+         
     def failureReport(self, errobj):
         """failure output for failovers from urlgrabber"""
         
@@ -81,8 +86,7 @@ class YumOutput:
     def listPkgs(self, lst, description, outputType):
         """outputs based on whatever outputType is. Current options:
            'list' - simple pkg list
-           'info' - similar to rpm -qi output
-           'rss' - rss feed-type output"""
+           'info' - similar to rpm -qi output"""
         
         if outputType in ['list', 'info']:
             thingslisted = 0
@@ -101,126 +105,21 @@ class YumOutput:
             if thingslisted == 0:
                 return 1, ['No Packages to list']
         
-        elif outputType == 'rss':
-            # take recent updates only and dump to an rss compat output
-            if len(lst) > 0:
-                if self.conf.getConfigOption('rss-filename') is None:
-                    raise yum.Errors.YumBaseError, \
-                       "No File specified for rss create"
-                else:
-                    fn = self.conf.getConfigOption('rss-filename')
-    
-                if fn[0] != '/':
-                    cwd = os.getcwd()
-                    fn = os.path.join(cwd, fn)
-                try:
-                    fo = open(fn, 'w')
-                except IOError, e:
-                    raise yum.Errors.YumBaseError, \
-                       "Could not open file %s for rss create" % (e)
-
-                doc = libxml2.newDoc('1.0')
-                self.xmlescape = doc.encodeEntitiesReentrant
-                rss = doc.newChild(None, 'rss', None)
-                rss.setProp('version', '2.0')
-                node = rss.newChild(None, 'channel', None)
-                rssheader = self.startRSS(description)
-                fo.write(rssheader)
-                for pkg in lst:
-                    item = self.rssnode(node, pkg)
-                    fo.write(item.serialize("utf-8", 1))
-                    item.unlinkNode()
-                    item.freeNode()
-                    del item
-                
-                end = self.endRSS()
-                fo.write(end)
-                fo.close()
-                del fo
-                doc.freeDoc()
-                del doc
-                
-    def startRSS(self, description='Yum Package List'):
-        """return string representation of rss preamble"""
-    
-        rfc822_format = "%a, %d %b %Y %X GMT"
-        now = time.strftime(rfc822_format, time.gmtime())
-        rssheader = """<?xml version="1.0" encoding="utf-8"?>
-    <rss version="2.0">
-      <channel>
-        <title>%s</title>
-        <link>http://linux.duke.edu/projects/yum/</link>
-        <description>%s</description>
-        <pubDate>%s</pubDate>
-        <generator>Yum</generator>
-        """ % (description, description, now)
-        
-        return rssheader
-    
-    def rssnode(self, node, pkg):
-        """return an rss20 compliant item node
-           takes a node, and a pkg object"""
-        
-        repo = self.repos.getRepo(pkg.repoid)
-        url = repo.urls[0]
-        rfc822_format = "%a, %d %b %Y %X GMT"
-        clog_format = "%a, %d %b %Y GMT"
-        xhtml_ns = "http://www.w3.org/1999/xhtml"
-        escape = self.xmlescape
-        
-        item = node.newChild(None, 'item', None)
-        title = escape(str(pkg))
-        item.newChild(None, 'title', title)
-        date = time.gmtime(float(pkg.returnSimple('buildtime')))
-        item.newChild(None, 'pubDate', time.strftime(rfc822_format, date))
-        item.newChild(None, 'guid', pkg.returnSimple('id'))
-        link = url + '/' + pkg.returnSimple('relativepath')
-        item.newChild(None, 'link', escape(link))
-
-        # build up changelog
-        changelog = ''
-        cnt = 0
-        for e in pkg.returnChangelog():
-            cnt += 1
-            if cnt > 3: 
-                changelog += '...'
-                break
-            (date, author, desc) = e
-            date = time.strftime(clog_format, time.gmtime(float(date)))
-            changelog += '%s - %s\n%s\n\n' % (date, author, desc)
-        body = item.newChild(None, "body", None)
-        body.newNs(xhtml_ns, None)
-        body.newChild(None, "p", escape(pkg.returnSimple('summary')))
-        body.newChild(None, "pre", escape(pkg.returnSimple('description')))
-        body.newChild(None, "p", 'Change Log:')
-        body.newChild(None, "pre", escape(changelog))
-        description = '<pre>%s - %s\n\n' % (escape(pkg.name), 
-                                            escape(pkg.returnSimple('summary')))
-        description += '%s\n\nChange Log:\n\n</pre>' % escape(pkg.returnSimple('description'))
-        description += escape('<pre>%s</pre>' % escape(changelog))
-        item.newChild(None, 'description', description)
-        
-        return item
-        
-    
-    def endRSS(self):
-        """end the rss output"""
-        end="\n  </channel>\n</rss>\n"
-        return end
     
         
     def userconfirm(self):
         """gets a yes or no from the user, defaults to No"""
 
-        while 1:            
+        while True:            
             choice = raw_input('Is this ok [y/N]: ')
-            if len(choice) == 0 or choice[0] in ['Y', 'y', 'N','n']:
+            choice = choice.lower()
+            if len(choice) == 0 or choice[0] in ['y', 'n']:
                 break
 
-        if len(choice) == 0 or choice[0] not in ['y', 'Y']:
-            return 0
+        if len(choice) == 0 or choice[0] != 'y':
+            return False
         else:            
-            return 1
+            return True
                 
     
     def displayPkgsInGroups(self, group):
@@ -254,7 +153,28 @@ class YumOutput:
             for item in self.groupInfo.optional_pkgs[groupid]:
                 print '   %s' % item
 
-           
+
+    def depListOutput(self, results):
+        """take a list of findDeps results and 'pretty print' the output"""
+        
+        for pkg in results.keys():
+            print "package: %s" % pkg.compactPrint()
+            if len(results[pkg].keys()) == 0:
+                print "  No dependencies for this package"
+                continue
+
+            for req in results[pkg].keys():
+                reqlist = results[pkg][req] 
+                print "  dependency: %s" % pkg.prcoPrintable(req)
+                if not reqlist:
+                    print "   Unsatisfied dependency"
+                    continue
+                
+                for po in reqlist:
+                    print "   provider: %s" % po.compactPrint()
+
+
+        
     def format_number(self, number, SI=0, space=' '):
         """Turn numbers into human-readable metric-like numbers"""
         symbols = ['',  # (none)
@@ -302,7 +222,7 @@ class YumOutput:
         for item in values:
             self.log(2, '%s' % item)
 
-    def reportDownloadSize(self,packages):
+    def reportDownloadSize(self, packages):
         """Report the total download size for a set of packages"""
         totsize = 0
         error = False
@@ -322,67 +242,69 @@ class YumOutput:
             self.log(1, "Total download size: %s" % (self.format_number(totsize)))
             
     def listTransaction(self):
-        """displays the transaction in an easy-to-read way."""
+        """returns a string rep of the  transaction in an easy-to-read way."""
         
-        out = ''
-        userout = ''
-        depout = ''
-        otherout = ''
-        
-        updated, installed, removed, obsoleted, depup, depinst, deprem = self.tsInfo.makelists()
-
-        for (action, pkglist) in [('Install', installed), ('Update', updated)]:
+        self.tsInfo.makelists()
+        if len(self.tsInfo) > 0:
+            out = """
+=============================================================================
+ %-22s  %-9s  %-15s  %-16s  %-5s
+=============================================================================
+""" % ('Package', 'Arch', 'Version', 'Repository', 'Size')
+        else:
+            out = ""
 
+        for (action, pkglist) in [('Installing', self.tsInfo.installed),
+                            ('Updating', self.tsInfo.updated),
+                            ('Removing', self.tsInfo.removed),
+                            ('Installing for dependencies', self.tsInfo.depinstalled),
+                            ('Updating for dependencies', self.tsInfo.depupdated),
+                            ('Removing for dependencies', self.tsInfo.depremoved)]:
+            if pkglist:
+                totalmsg = "%s:\n" % action
             for txmbr in pkglist:
                 (n,a,e,v,r) = txmbr.pkgtup
-                msg = "  %s: %s.%s %s:%s-%s - %s\n" % (action, n,a,e,v,r, txmbr.repoid)
-                userout = userout + msg
+                evr = txmbr.po.printVer()
+                repoid = txmbr.repoid
+                pkgsize = float(txmbr.po.size())
+                size = self.format_number(pkgsize)
+                msg = " %-22s  %-9s  %-15s  %-16s  %5s\n" % (n, a,
+                              evr, repoid, size)
+                for (obspo, relationship) in txmbr.relatedto:
+                    if relationship == 'obsoletes':
+                        appended = '     replacing  %s.%s %s\n\n' % (obspo.name,
+                            obspo.arch, obspo.printVer())
+                        msg = msg+appended
+                totalmsg = totalmsg + msg
         
-        for (action, pkglist) in [('Remove', removed)]:
-            for txmbr in pkglist:
-                (n,a,e,v,r) = txmbr.pkgtup
-                msg = "  %s: %s.%s %s:%s-%s\n" % (action, n,a,e,v,r)
-                userout = userout + msg
+            if pkglist:
+                out = out + totalmsg
+
+        summary = """
+Transaction Summary
+=============================================================================
+Install  %5.5s Package(s)         
+Update   %5.5s Package(s)         
+Remove   %5.5s Package(s)         
+""" % (len(self.tsInfo.installed + self.tsInfo.depinstalled),
+       len(self.tsInfo.updated + self.tsInfo.depupdated),
+       len(self.tsInfo.removed + self.tsInfo.depremoved))
+        out = out + summary
         
-        for (action, pkglist) in [('Install', depinst), ('Update', depup)]:
-
-            for txmbr in pkglist:
-                (n,a,e,v,r) = txmbr.pkgtup
-                msg = "  %s: %s.%s %s:%s-%s - %s\n" % (action, n,a,e,v,r, txmbr.repoid)
-                depout = depout + msg
-                   
-        for (action, pkglist) in [('Remove', deprem)]:
-            for txmbr in pkglist:
-                (n,a,e,v,r) = txmbr.pkgtup
-                msg = "  %s: %s.%s %s:%s-%s\n" % (action, n,a,e,v,r)
-                depout = depout + msg
-
-        for txmbr in obsoleted:
-            (n,a,e,v,r) = txmbr.pkgtup
-            obspkg = None
-            for (pkg, relationship) in txmbr.relatedto:
-                if relationship == 'osboletedby':
-                    obspkg = '%s.%s %s:%s-%s' % pkg
-            if obspkg is not None:
-                otherout = otherout + "  Obsoleting: %s.%s %s:%s-%s with %s\n" % (n, a, e, v, r, obspkg)
-
-        out = "Transaction Listing:\n%s" % userout 
-        if depout != '':
-            out = out + "\nPerforming the following to resolve dependencies:\n%s" % depout
-        if otherout != '':
-            out = out + "\nOther Transactions:\n%s\n" % otherout
-              
         return out
-
+        
     def postTransactionOutput(self):
         out = ''
         
-        updated, installed, removed, obsoleted, depup, depinst, deprem = self.tsInfo.makelists()
+        self.tsInfo.makelists()
 
-        for (action, pkglist) in [('Removed', removed), ('Dependency Removed', deprem),
-                                  ('Installed', installed), ('Dependency Installed', depinst),
-                                  ('Updated', updated), ('Dependency Updated', depup),
-                                  ('Obsoleted', obsoleted)]:
+        for (action, pkglist) in [('Removed', self.tsInfo.removed), 
+                                  ('Dependency Removed', self.tsInfo.depremoved),
+                                  ('Installed', self.tsInfo.installed), 
+                                  ('Dependency Installed', self.tsInfo.depinstalled),
+                                  ('Updated', self.tsInfo.updated),
+                                  ('Dependency Updated', self.tsInfo.depupdated),
+                                  ('Replaced', self.tsInfo.obsoleted)]:
             
             if len(pkglist) > 0:
                 out += '\n%s:' % action
@@ -393,7 +315,28 @@ class YumOutput:
         
         return out
 
+    def setupProgessCallbacks(self):
+        """sets up the progress callbacks and various 
+           output bars based on debug level"""
 
+        # if we're below 2 on the debug level we don't need to be outputting
+        # progress bars - this is hacky - I'm open to other options
+        # One of these is a download
+        if self.conf.debuglevel < 2 or not sys.stdout.isatty():
+            self.repos.setProgressBar(None)
+            self.repos.callback = None
+        else:
+            self.repos.setProgressBar(TextMeter(fo=sys.stdout))
+            self.repos.callback = CacheProgressCallback(self.log, self.errorlog,
+                                                        self.filelog)
+        # setup our failure report for failover
+        freport = (self.failureReport,(),{})
+        self.repos.setFailureCallback(freport)
+        
+        # setup our depsolve progress callback
+        dscb = DepSolveProgressCallBack(self.log, self.errorlog)
+        self.dsCallback = dscb
+            
     
     def pickleRecipe(self):
         """ don't ask """
