@@ -55,6 +55,11 @@ class PackageSack:
             elif failure == 'build':
                 self.buildIndexes()
 
+    def packagesByTuple(self, pkgtup):
+        """return a list of package objects by (n,a,e,v,r) tuple"""
+        (n,a,e,v,r) = pkgtup
+        return self.searchNevra(name=n, arch=a, epoch=e, ver=v, rel=r)
+        
     def searchNevra(self, name=None, epoch=None, ver=None, rel=None, arch=None):
         """return list of pkgobjects matching the nevra requested"""
         self._checkIndexes(failure='build')
@@ -296,7 +301,7 @@ class PackageSack:
         for pkg in self.returnPackages():
             print pkg.returnNevraPrintable()
 
-    def excludeArches(self, archlist):
+    def excludeArchs(self, archlist):
         """exclude incompatible arches. archlist is a list of compatible arches"""
         
         for pkg in self.returnPackages():
