@@ -12,7 +12,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-# Copyright 2004 Duke University
+# Copyright 2005 Duke University
 
 
 import os
@@ -27,6 +27,7 @@ import time
 import sre_constants
 import glob
 
+
 import Errors
 import rpmUtils
 import rpmUtils.updates
@@ -34,12 +35,19 @@ import rpmUtils.arch
 import groups
 import config
 import repos
+import misc
 import transactioninfo
 from urlgrabber.grabber import URLGrabError
 import depsolve
+import plugins
 
-from packages import parsePackages, YumLocalPackage, YumInstalledPackage
+
+from packages import parsePackages, YumLocalPackage, YumInstalledPackage, bestPackage
 from repomd import mdErrors
+from constants import *
+from repomd.packageSack import ListPackageSack
+
+__version__ = '2.4.0'
 
 class YumBase(depsolve.Depsolve):
     """This is a primary structure and base class. It houses the objects and
@@ -53,7 +61,10 @@ class YumBase(depsolve.Depsolve):
         self.repos = repos.RepoStorage() # class of repositories
         if (not self.repos.sqlite):
             self.log(1,"Warning, could not load sqlite, falling back to pickle")
-    
+
+        # Start with plugins disabled
+        self.disablePlugins()
+
     def log(self, value, msg):
         """dummy log stub"""
         print msg
@@ -64,10 +75,10 @@ class YumBase(depsolve.Depsolve):
 
     def filelog(self, value, msg):
         print msg
-    
+   
     def doConfigSetup(self, fn='/etc/yum.conf', root='/'):
         """basic stub function for doing configuration setup"""
-        
+       
         self.conf = config.yumconf(configfile=fn, root=root)
         self.getReposFromConfig()
 
@@ -86,36 +97,35 @@ class YumBase(depsolve.Depsolve):
                 else:
                     reposlist.append(thisrepo)
 
-        # reads through reposdir for *.repo
+        # reads through each reposdir for *.repo
         # does not read recursively
         # read each of them in using confpp, then parse them same as any other repo
         # section - as above.
-        reposdir = self.conf.reposdir
-        if os.path.exists(self.conf.installroot + '/' + reposdir):
-            reposdir = self.conf.installroot + '/' + reposdir
-        
-        reposglob = reposdir + '/*.repo'
-        if os.path.exists(reposdir) and os.path.isdir(reposdir):
-            repofn = glob.glob(reposglob)
-            repofn.sort()
+        for reposdir in self.conf.reposdir:
+            if os.path.exists(self.conf.installroot + '/' + reposdir):
+                reposdir = self.conf.installroot + '/' + reposdir
             
-            for fn in repofn:
-                if not os.path.isfile(fn):
-                    continue
-                try:
-                    cfg, sections = config.parseDotRepo(fn)
-                except Errors.ConfigError, e:
-                    self.errorlog(2, e)
-                    continue
-
-                for section in sections:
+            if os.path.isdir(reposdir):
+                repofn = glob.glob(reposdir+'/*.repo')
+                repofn.sort()
+                
+                for fn in repofn:
+                    if not os.path.isfile(fn):
+                        continue
                     try:
-                        thisrepo = config.cfgParserRepo(section, self.conf, cfg)
-                    except (Errors.RepoError, Errors.ConfigError), e:
+                        cfg, sections = config.parseDotRepo(fn)
+                    except Errors.ConfigError, e:
                         self.errorlog(2, e)
                         continue
-                    else:
-                        reposlist.append(thisrepo)
+
+                    for section in sections:
+                        try:
+                            thisrepo = config.cfgParserRepo(section, self.conf, 
+                                    cfg)
+                            reposlist.append(thisrepo)
+                        except (Errors.RepoError, Errors.ConfigError), e:
+                            self.errorlog(2, e)
+                            continue
 
         # got our list of repo objects
         for thisrepo in reposlist:
@@ -124,7 +134,34 @@ class YumBase(depsolve.Depsolve):
             except Errors.RepoError, e: 
                 self.errorlog(2, e)
                 continue
-        
+
+    def disablePlugins(self):
+        '''Disable yum plugins
+        '''
+        self.plugins = plugins.DummyYumPlugins()
+    
+    def doPluginSetup(self, optparser=None, types=None):
+        '''Initialise and enable yum plugins. 
+
+        If plugins are going to be used, this should be called soon after
+        doConfigSetup() has been called.
+
+        @param optparser: The OptionParser instance for this run (optional)
+        @param types: A sequence specifying the types of plugins to load.
+            This should be sequnce containing one or more of the
+            yum.plugins.TYPE_...  constants. If None (the default), all plugins
+            will be loaded.
+        '''
+        # Load plugins first as they make affect available config options
+        self.plugins = plugins.YumPlugins(self, self.conf.pluginpath,
+                optparser, types)
+
+        # Process options registered by plugins
+        self.plugins.parseopts(self.conf, self.repos.findRepos('*'))
+
+        # Initialise plugins
+        self.plugins.run('init')
+
     def doTsSetup(self):
         """setup all the transaction set storage items we'll need
            This can't happen in __init__ b/c we don't know our installroot
@@ -164,33 +201,53 @@ class YumBase(depsolve.Depsolve):
         if hasattr(self, 'up'):
             del self.up
 
-    def doRepoSetup(self, nosack=None):
-        """grabs the repomd.xml for each enabled repository and sets up the basics
-           of the repository - stub"""
-           
-        for repo in self.repos.listEnabled():
+    def doRepoSetup(self, thisrepo=None):
+        """grabs the repomd.xml for each enabled repository and sets up 
+           the basics of the repository"""
+
+        
+        self.plugins.run('prereposetup')
+        
+        repos = []
+        if thisrepo is None:
+            repos = self.repos.listEnabled()
+        else:
+            repos = self.repos.findRepos(thisrepo)
+
+        if len(repos) < 1:
+            self.errorlog(0, 'No Repositories Available to Set Up')
+
+        for repo in repos:
             if repo.repoXML is not None and len(repo.urls) > 0:
                 continue
             try:
                 repo.cache = self.conf.getConfigOption('cache')
                 repo.baseurlSetup()
                 repo.dirSetup()
+                self.log(3, 'Baseurl(s) for repo: %s' % repo.urls)
             except Errors.RepoError, e:
                 self.errorlog(0, '%s' % e)
                 raise
                 
             try:
-                repo.getRepoXML()
+                repo.getRepoXML(text=repo)
             except Errors.RepoError, e:
                 self.errorlog(0, 'Cannot open/read repomd.xml file for repository: %s' % repo)
                 self.errorlog(0, str(e))
                 raise
 
+        self.plugins.run('postreposetup')
 
-    def doSackSetup(self, archlist=None):
+    def doSackSetup(self, archlist=None, thisrepo=None):
         """populates the package sacks for information from our repositories,
            takes optional archlist for archs to include"""
            
+        
+        if thisrepo is None:
+            repos = self.repos.listEnabled()
+        else:
+            repos = self.repos.findRepos(thisrepo)
+            
         self.log(3, 'Setting up Package Sacks')
         if not archlist:
             archlist = rpmUtils.arch.getArchList()
@@ -200,13 +257,14 @@ class YumBase(depsolve.Depsolve):
             archdict[arch] = 1
 
         self.repos.pkgSack.compatarchs = archdict
-        self.repos.populateSack()
+        self.repos.populateSack(which=repos)
         self.pkgSack = self.repos.pkgSack
         self.excludePackages()
         self.excludeNonCompatArchs(archlist=archlist)
-        for repo in self.repos.listEnabled():
+        for repo in repos:
             self.excludePackages(repo)
             self.includePackages(repo)
+        self.plugins.run('exclude')
         self.pkgSack.buildIndexes()
         
     def doUpdateSetup(self):
@@ -228,6 +286,7 @@ class YumBase(depsolve.Depsolve):
             self.up.rawobsoletes = self.pkgSack.returnObsoletes()
             
         self.up.exactarch = self.conf.getConfigOption('exactarch')
+        self.up.exactarchlist = self.conf.getConfigOption('exactarchlist')
         self.up.doUpdates()
 
         if self.conf.getConfigOption('obsoletes'):
@@ -276,9 +335,26 @@ class YumBase(depsolve.Depsolve):
     def buildTransaction(self):
         """go through the packages in the transaction set, find them in the
            packageSack or rpmdb, and pack up the ts accordingly"""
+        self.plugins.run('preresolve')
         (rescode, restring) = self.resolveDeps()
+        self.plugins.run('postresolve', rescode=rescode, restring=restring)
         return rescode, restring
 
+    def runTransaction(self, cb):
+        """takes an rpm callback object, performs the transaction"""
+
+        self.plugins.run('pretrans')
+
+        errors = self.ts.run(cb.callback, '')
+        if errors:
+            errstring = 'Error in Transaction: '
+            for descr in errors:
+                errstring += '  %s\n' % str(descr)
+            
+            raise Errors.YumBaseError, errstring
+
+        self.plugins.run('posttrans')
+        
     def excludePackages(self, repo=None):
         """removes packages from packageSacks based on global exclude lists,
            command line excludes and per-repository excludes, takes optional 
@@ -303,12 +379,12 @@ class YumBase(depsolve.Depsolve):
             self.log(2, 'Excluding Packages from %s' % repo.name)
             
         exactmatch, matched, unmatched = \
-           parsePackages(self.pkgSack.returnPackages(repoid), excludelist)
+           parsePackages(self.pkgSack.returnPackages(repoid), excludelist, casematch=1)
         
         for po in exactmatch + matched:
             self.log(3, 'Excluding %s' % po)
             self.pkgSack.delPackage(po)
-        
+      
         self.log(2, 'Finished')
 
     def includePackages(self, repo):
@@ -322,7 +398,7 @@ class YumBase(depsolve.Depsolve):
         
         pkglist = self.pkgSack.returnPackages(repo.id)
         exactmatch, matched, unmatched = \
-           parsePackages(pkglist, includelist)
+           parsePackages(pkglist, includelist, casematch=1)
         
         self.log(2, 'Reducing %s to included packages only' % repo.name)
         rmlist = []
@@ -358,6 +434,10 @@ class YumBase(depsolve.Depsolve):
         # if we're not root then we don't lock - just return nicely
         if self.conf.getConfigOption('uid') != 0:
             return
+            
+        root = self.conf.installroot
+        lockfile = root + '/' + lockfile # lock in the chroot
+        lockfile = os.path.normpath(lockfile) # get rid of silly preceding extra /
         
         mypid=str(os.getpid())    
         while not self._lock(lockfile, mypid, 0644):
@@ -387,11 +467,18 @@ class YumBase(depsolve.Depsolve):
         # if we're not root then we don't lock - just return nicely
         if self.conf.getConfigOption('uid') != 0:
             return
+        
+        root = self.conf.installroot
+        lockfile = root + '/' + lockfile # lock in the chroot
+        
         self._unlock(lockfile)
         
     def _lock(self, filename, contents='', mode=0777):
+        lockdir = os.path.dirname(filename)
         try:
-            fd = os.open(filename, os.O_EXCL|os.O_CREAT|os.O_WRONLY, mode)
+            if not os.path.exists(lockdir):
+                os.makedirs(lockdir, mode=0755)
+            fd = os.open(filename, os.O_EXCL|os.O_CREAT|os.O_WRONLY, mode)    
         except OSError, msg:
             if not msg.errno == errno.EEXIST: raise msg
             return 0
@@ -412,7 +499,7 @@ class YumBase(depsolve.Depsolve):
            raiseError  = defaults to 0 - if 1 then will raise
            a URLGrabError if the file does not check out.
            otherwise it returns false for a failure, true for success"""
-        
+
         if type(fo) is types.InstanceType:
             fo = fo.filename
 
@@ -458,6 +545,9 @@ class YumBase(depsolve.Depsolve):
         """download list of package objects handed to you, output based on
            callback, raise yum.Errors.YumBaseError on problems"""
 
+        errors = {}
+        self.plugins.run('predownload', pkglist=pkglist)
+        repo_cached = False
         remote_pkgs = []
         for po in pkglist:
             if hasattr(po, 'pkgtype') and po.pkgtype == 'local':
@@ -469,9 +559,18 @@ class YumBase(depsolve.Depsolve):
                 totsize = int(po.size())
                 try:
                     result = self.verifyPkg(local, po, raiseError=1)
-                except URLGrabError, e:
+                except URLGrabError, e: # fails the check
+                    
+                    repo = self.repos.getRepo(po.repoid)
+                    if repo.cache:
+                        repo_cached = True
+                        msg = 'package fails checksum but caching is enabled for %s' % repo.id
+                        if not errors.has_key(po): errors[po] = []
+                        errors[po].append(msg)
+                        
                     if cursize >= totsize: # keep it around for regetting
                         os.unlink(local)
+                        
                 else:
                     if result:
                         continue
@@ -479,8 +578,15 @@ class YumBase(depsolve.Depsolve):
                         if cursize >= totsize: # keep it around for regetting
                             os.unlink(local)
             remote_pkgs.append(po)
+            
+            # caching is enabled and the package 
+            # just failed to check out there's no 
+            # way to save this, report the error and return
+            if (self.conf.cache or repo_cached) and errors:
+                return errors
+                
+
 
-        errors = {}
         i = 0
         for po in remote_pkgs:
             i += 1
@@ -495,7 +601,9 @@ class YumBase(depsolve.Depsolve):
                 mylocal = repo.get(relative=remote,
                                    local=local,
                                    checkfunc=checkfunc,
-                                   text=text)
+                                   text=text,
+                                   cache=repo.http_caching != 'none',
+                                   )
             except Errors.RepoError, e:
                 if not errors.has_key(po):
                     errors[po] = []
@@ -505,6 +613,8 @@ class YumBase(depsolve.Depsolve):
                 if errors.has_key(po):
                     del errors[po]
 
+        self.plugins.run('postdownload', pkglist=pkglist, errors=errors)
+
         return errors
 
     def verifyHeader(self, fo, po, raiseError):
@@ -566,8 +676,10 @@ class YumBase(depsolve.Depsolve):
         
         try:
             checkfunc = (self.verifyHeader, (po, 1), {})
-            hdrpath = repo.get(relative=remote, local=local, start=start, reget=None, 
-                               end=end, checkfunc=checkfunc, copy_local=1)
+            hdrpath = repo.get(relative=remote, local=local, start=start,
+                    reget=None, end=end, checkfunc=checkfunc, copy_local=1,
+                    cache=repo.http_caching != 'none',
+                    )
         except Errors.RepoError, e:
             saved_repo_error = e
             try:
@@ -609,11 +721,11 @@ class YumBase(depsolve.Depsolve):
                     result = 1
                 else:
                     result = 2
-                msg = 'public key not available for %s' % localfn
+                msg = 'Public key for %s is not installed' % localfn
 
             elif sigresult == 2:
                 result = 2
-                msg = 'problem opening package %s' % localfn
+                msg = 'Problem opening package %s' % localfn
 
             elif sigresult == 3:
                 if hasgpgkey:
@@ -621,11 +733,11 @@ class YumBase(depsolve.Depsolve):
                 else:
                     result = 2
                 result = 1
-                msg = 'untrusted public key for %s' % localfn
+                msg = 'Public key for %s is not trusted' % localfn
 
             elif sigresult == 4:
                 result = 2 
-                msg = 'unsigned package %s' % localfn
+                msg = 'Package %s is not signed' % localfn
             
         else:
             result =0
@@ -887,14 +999,54 @@ class YumBase(depsolve.Depsolve):
     def _refineSearchPattern(self, arg):
         """Takes a search string from the cli for Search or Provides
            and cleans it up so it doesn't make us vomit"""
-           
+        
         if re.match('.*[\*,\[,\],\{,\},\?,\+].*', arg):
             restring = fnmatch.translate(arg)
         else:
-            restring = arg
-        
+            restring = re.escape(arg)
+            
         return restring
         
+    def findDeps(self, pkgs):
+        """Return the dependencies for a given package, as well
+           possible solutions for those dependencies.
+           
+           Returns the deps as a dict  of:
+             packageobject = [reqs] = [list of satisfying pkgs]"""
+        
+        results = {}
+        self.doRepoSetup()
+        self.doRpmDBSetup()
+
+        avail = self.pkgSack.returnPackages()
+        exactmatch, matched, unmatched = parsePackages(avail, pkgs)
+
+        if len(unmatched) > 0:
+            self.errorlog(0, 'No Match for arguments: %s' % unmatched)
+
+        pkgs = misc.unique(exactmatch + matched)
+        
+        for pkg in pkgs:
+            results[pkg] = {} 
+            reqs = pkg.returnPrco('requires');
+            reqs.sort()
+            pkgresults = results[pkg] # shorthand so we don't have to do the
+                                      # double bracket thing
+            
+            for req in reqs:
+                (r,f,v) = req
+                if r.startswith('rpmlib('):
+                    continue
+                
+                satisfiers = []
+
+                for po in self.whatProvides(r, f, v):
+                    satisfiers.append(po)
+
+                pkgresults[req] = satisfiers
+        
+        return results
+    
     def searchPackages(self, fields, criteria, callback=None):
         """Search specified fields for matches to criteria
            optional callback specified to print out results
@@ -917,7 +1069,7 @@ class YumBase(depsolve.Depsolve):
                 tmpvalues = []
                 for field in fields:
                     value = po.returnSimple(field)
-                    if crit_re.search(value):
+                    if value and crit_re.search(value):
                         tmpvalues.append(value)
                 if len(tmpvalues) > 0:
                     if callback:
@@ -940,7 +1092,7 @@ class YumBase(depsolve.Depsolve):
                     value = po.returnSimple(field)
                     if type(value) is types.ListType: # this is annoying
                         value = str(value)
-                    if crit_re.search(value):
+                    if value and crit_re.search(value):
                         tmpvalues.append(value)
             if len(tmpvalues) > 0:
                 if callback:
@@ -955,6 +1107,14 @@ class YumBase(depsolve.Depsolve):
         self.doRpmDBSetup()
         matches = {}
         
+        # search deps the simple way first
+        for arg in args:
+            pkgs = self.returnPackagesByDep(arg)
+            for po in pkgs:
+                if callback:
+                    callback(po, [arg])
+                matches[po] = [arg]
+
         # search pkgSack - fully populate the worthwhile metadata to search
         # if it even vaguely matches
         for arg in args:
@@ -970,17 +1130,17 @@ class YumBase(depsolve.Depsolve):
 
         for arg in args:
             restring = self._refineSearchPattern(arg)
-            try: arg_re = re.compile(restring, flags=re.I)
+            try: 
+                arg_re = re.compile(restring, flags=re.I)
             except sre_constants.error, e:
                 raise Errors.MiscError, \
-                 'Search Expression: %s is an invalid Regular Expression.\n' % arg
+                  'Search Expression: %s is an invalid Regular Expression.\n' % arg
             
             # If this is not a regular expression, only search in packages
             # returned by pkgSack.searchAll
-            if restring.find('*') == restring.find('?') \
-              == restring.find('%') == -1 and \
+            if arg.find('*') == arg.find('?')  == arg.find('%') == -1 and \
               hasattr(self.pkgSack,'searchAll'):
-                where = self.pkgSack.searchAll(restring)
+                where = self.pkgSack.searchAll(arg)
             else:
                 where = self.pkgSack
 
@@ -1062,6 +1222,8 @@ class YumBase(depsolve.Depsolve):
             else:
                 available.append(grp.name)
 
+        installed = misc.unique(installed)
+        available = misc.unique(available)
         installed.sort()
         available.sort()
         
@@ -1081,10 +1243,10 @@ class YumBase(depsolve.Depsolve):
             if po.pkgtup == pkgtup:
                 return po
                 
-        pkgs = self.pkgSack.searchNevra(name=n, arch=a, epoch=e, ver=v, rel=r)
+        pkgs = self.pkgSack.packagesByTuple(pkgtup)
 
         if len(pkgs) == 0:
-            raise Errors.DepError, 'Package tuple %s could not be found in packagesack' % pkgtup
+            raise Errors.DepError, 'Package tuple %s could not be found in packagesack' % str(pkgtup)
             return None
             
         if len(pkgs) > 1: # boy it'd be nice to do something smarter here FIXME
@@ -1097,6 +1259,14 @@ class YumBase(depsolve.Depsolve):
         
         return result
 
+    def getInstalledPackageObject(self, pkgtup):
+        """returns a YumInstallPackage object for the pkgtup specified"""
+        
+        hdrs = self.rpmdb.returnHeaderByTuple(pkgtup)
+        hdr = hdrs[0]
+        po = YumInstalledPackage(hdr)
+        return po
+        
     def gpgKeyCheck(self):
         """checks for the presence of gpg keys in the rpmdb
            returns 0 if no keys returns 1 if keys"""
@@ -1115,16 +1285,20 @@ class YumBase(depsolve.Depsolve):
         if keys == 0:
             return 0
         else:
+            mydir = os.path.dirname(gpgkeyschecked)
+            if not os.path.exists(mydir):
+                os.makedirs(mydir)
+                
             fo = open(gpgkeyschecked, 'w')
             fo.close()
             del fo
             return 1
 
-    def returnPackageByDep(self, depstring):
+    def returnPackagesByDep(self, depstring):
         """Pass in a generic [build]require string and this function will 
-           pass back the best(or first) package it finds providing that dep."""
+           pass back the packages it finds providing that dep."""
         
-        flags = {'>':'GT', '<':'LT', '=': 'EQ', '==': 'EQ', '>=':'GE', '<=':'LE'}
+        results = []
         self.doRepoSetup()
         # parse the string out
         #  either it is 'dep (some operator) e:v-r'
@@ -1137,22 +1311,51 @@ class YumBase(depsolve.Depsolve):
         if depstring[0] != '/':
             # not a file dep - look at it for being versioned
             if re.search('[>=<]', depstring):  # versioned
-                depname, flagsymbol, depver = depstring.split()
-                if not flags.has_key(flagsymbol):
-                    raise Errors.YumBaseError, 'No Packages found for %s' % depstring
-                depflags = flags[flagsymbol]
+                try:
+                    depname, flagsymbol, depver = depstring.split()
+                except ValueError, e:
+                    raise Errors.YumBaseError, 'Invalid versioned dependency string, try quoting it.'
+                if not SYMBOLFLAGS.has_key(flagsymbol):
+                    raise Errors.YumBaseError, 'Invalid version flag'
+                depflags = SYMBOLFLAGS[flagsymbol]
                 
         sack = self.whatProvides(depname, depflags, depver)
-        if len(sack) < 1:
-            raise Errors.YumBaseError, 'No Packages found for %s' % depstring
+        results = sack.returnPackages()
+        return results
         
-        if len(sack) == 1:
-            for po in sack:
-                return po
+
+    def returnPackageByDep(self, depstring):
+        """Pass in a generic [build]require string and this function will 
+           pass back the best(or first) package it finds providing that dep."""
         
-        pkglist = sack.returnNewestByNameArch()
-        best = pkglist[0]
-        for pkg in pkglist[1:]:
+        try:
+            pkglist = self.returnPackagesByDep(depstring)
+        except Errors.YumBaseError, e:
+            raise Errors.YumBaseError, 'No Package found for %s' % depstring
+        
+        result = self.bestPackageFromList(pkglist)
+        if result is None:
+            raise Errors.YumBaseError, 'No Package found for %s' % depstring
+        
+        return result
+        
+    def bestPackageFromList(self, pkglist):
+        """take list of package objects and return the best package object.
+           If the list is empty, raise Errors.YumBaseError"""
+        
+        
+        if len(pkglist) == 0:
+            return None
+            
+        if len(pkglist) == 1:
+            return pkglist[0]
+        
+        mysack = ListPackageSack()
+        mysack.addList(pkglist)
+        bestlist = mysack.returnNewestByNameArch() # get rid of all lesser vers
+        
+        best = bestlist[0]
+        for pkg in bestlist[1:]:
             if len(pkg.name) < len(best.name): # shortest name silliness
                 best = pkg
                 continue
@@ -1164,5 +1367,38 @@ class YumBase(depsolve.Depsolve):
             if arch == pkg.arch:
                 best = pkg
                 continue
-        
+
         return best
+
+    def returnInstalledPackagesByDep(self, depstring):
+        """Pass in a generic [build]require string and this function will 
+           pass back the installed packages it finds providing that dep."""
+        
+        results = []
+        self.doRpmDBSetup()
+        # parse the string out
+        #  either it is 'dep (some operator) e:v-r'
+        #  or /file/dep
+        #  or packagename
+        depname = depstring
+        depflags = None
+        depver = None
+        
+        if depstring[0] != '/':
+            # not a file dep - look at it for being versioned
+            if re.search('[>=<]', depstring):  # versioned
+                try:
+                    depname, flagsymbol, depver = depstring.split()
+                except ValueError, e:
+                    raise Errors.YumBaseError, 'Invalid versioned dependency string, try quoting it.'
+                if not SYMBOLFLAGS.has_key(flagsymbol):
+                    raise Errors.YumBaseError, 'Invalid version flag'
+                depflags = SYMBOLFLAGS[flagsymbol]
+                
+        pkglist = self.rpmdb.whatProvides(depname, depflags, depver)
+        
+        for pkgtup in pkglist:
+            results.append(self.getInstalledPackageObject(pkgtup))
+        
+        return results
+
