@@ -24,6 +24,7 @@ import urllib
 import glob
 import rpm
 import re
+import types
 
 import rpmUtils.transaction
 import rpmUtils.arch
@@ -31,7 +32,9 @@ import Errors
 import urlgrabber
 import urlgrabber.grabber
 from repos import variableReplace, Repository
+from constants import *
 
+HTTP_CACHING_VALS = ('none', 'packages', 'all')
 
 class CFParser(ConfigParser.ConfigParser):
     """wrapper around ConfigParser to provide two simple but useful functions:
@@ -54,7 +57,7 @@ class CFParser(ConfigParser.ConfigParser):
            """
         try:
             return self.getboolean(section, option)
-        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError), e:            
+        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError), e:
             return default
 
     def _getint(self, section, option, default=None):
@@ -77,6 +80,17 @@ class CFParser(ConfigParser.ConfigParser):
         except (ConfigParser.NoSectionError, ConfigParser.NoOptionError), e:            
             return default
 
+    def getselection(self, section, option, default=None, allowed=()):
+        '''Get a string value that is only allowed to take specific values
+
+        Will raise ValueError if the value given is not allowed.
+        '''
+        val = self._getoption(section, option, default)
+        if val not in allowed:
+            raise ValueError('"%s" is not a valid value for %s' % (val, 
+                option))
+        return val
+
     def getbytes(self, section, option, default=None):
         """Get a friendly bytes/bandwidth option as bytes. 
 
@@ -180,7 +194,6 @@ class yumconf(object):
         except ConfigParser.ParsingError, e:
             raise Errors.ConfigError, str(e)
             
-
         self.configdata = {} # dict to hold all the data goodies
        
         
@@ -193,9 +206,7 @@ class yumconf(object):
         #defaults -either get them or set them
         optionstrings = [('cachedir', '/var/cache/yum'), 
                          ('logfile', '/var/log/yum.log'), 
-                         ('reposdir', '/etc/yum.repos.d'),
-                         ('rss-filename', 'yum-rss.xml'),
-                         ('pkgpolicy', 'newest'),
+                         ('reposdir', ['/etc/yum/repos.d', '/etc/yum.repos.d']),
                          ('syslog_ident', None),
                          ('syslog_facility', 'LOG_USER'),
                          ('distroverpkg', 'fedora-release'),
@@ -207,24 +218,34 @@ class yumconf(object):
                          ('proxy', None),
                          ('proxy_username', None),
                          ('proxy_password', None),
+                         ('pluginpath', ['/usr/lib/yum-plugins']),
                          ('installonlypkgs', ['kernel', 'kernel-bigmem', 
                                               'kernel-enterprise','kernel-smp',
+                                              'kernel-modules',
                                               'kernel-debug', 'kernel-unsupported', 
                                               'kernel-source', 'kernel-devel']),
                          ('kernelpkgnames', ['kernel','kernel-smp',
                                              'kernel-enterprise', 'kernel-bigmem',
                                              'kernel-BOOT']),
-                         ('tsflags', [])]
+                         ('exactarchlist', ['kernel', 'kernel-smp', 'glibc',
+                                            'kernel-hugemem', 'kernel-enterprise',
+                                            'kernel-bigmem']),
+                         ('tsflags', []),
+                         ]
                          
-        optionbools = [('assumeyes', 0),
-                       ('exactarch', 1),
-                       ('tolerant', 1),
-                       ('diskspacecheck', 1),
-                       ('overwrite_groups', 0),
-                       ('keepalive', 1),
-                       ('gpgcheck', 0),
-                       ('obsoletes', 0),
-                       ('showdupesfromrepos', 0)]
+        optionbools = [('assumeyes', 'False'),
+                       ('alwaysprompt', 'True'),
+                       ('exactarch', 'True'),
+                       ('tolerant', 'True'),
+                       ('diskspacecheck', 'True'),
+                       ('overwrite_groups', 'False'),
+                       ('keepalive', 'True'),
+                       ('gpgcheck', 'False'),
+                       ('obsoletes', 'False'),
+                       ('showdupesfromrepos', 'False'),
+                       ('enabled', 'True'),
+                       ('plugins', 'False'),
+                       ('enablegroups', 'True')]
 
         # not being set from the config file
         # or things that can't be handled like all the rest
@@ -235,22 +256,58 @@ class yumconf(object):
 
         optionfloats = [('timeout', 30.0)]
 
-
-        # do the ints
-        for (option, default) in optionints:
-            value =  self.cfg._getint('main', option, default)
+        # do these two early so we can do the rest using variableReplace()
+        for (option, default) in [('distroverpkg' , 'fedora-release'),
+                                  ('installroot', root)]:
+            value =  self.cfg._getoption('main', option, default)
             self.configdata[option] = value
             setattr(self, option, value)
 
-        # do the strings        
+        # get our variables parsed
+        self.yumvar = self._getEnvVar()
+        self.yumvar['basearch'] = rpmUtils.arch.getBaseArch() # FIXME make this configurable??
+        self.yumvar['arch'] = rpmUtils.arch.getCanonArch() # FIXME make this configurable??
+        # figure out what the releasever really is from the distroverpkg
+        self.yumvar['releasever'] = self._getsysver()
+
+        # do the ints
+        for (option, default) in optionints:
+            try:
+                value =  self.cfg._getoption('main', option, default)
+                value = variableReplace(self.yumvar, value)
+                value = int(value)
+            except ValueError, e:
+                raise Errors.ConfigError, 'Invalid value in config for main::%s' % option
+            self.configdata[option] = value
+            setattr(self, option, value)
+
+        # do the floats
+        for (option, value) in optionfloats:
+            try:
+                value =  self.cfg._getoption('main', option, default)
+                value = variableReplace(self.yumvar, value)
+                value = float(value)
+            except ValueError, e:
+                raise Errors.ConfigError, 'Invalid value in config for main::%s' % option
+                
+            self.configdata[option] = value
+            setattr(self, option, value)
+
+        # do the strings
         for (option, default) in optionstrings:
             value =  self.cfg._getoption('main', option, default)
+            value = variableReplace(self.yumvar, value)
             self.configdata[option] = value
             setattr(self, option, value)
             
         # do the bools
+        
         for (option, default) in optionbools:
-            value = self.cfg._getboolean('main', option, default)
+            value = self.cfg._getoption('main', option, default)
+            value = variableReplace(self.yumvar, value)
+            if value.lower() not in BOOLEAN_STATES:
+                raise Errors.ConfigError, 'Invalid value in config for main::%s' % option
+            value = BOOLEAN_STATES[value.lower()]
             self.configdata[option] = value
             setattr(self, option, value)
             
@@ -259,13 +316,6 @@ class yumconf(object):
             self.configdata[option] = value
             setattr(self, option, value)
 
-        # do the floats
-        for (option, value) in optionfloats:
-            value = self.cfg._getfloat('main', option, default)
-            self.configdata[option] = value
-            setattr(self, option, value)
-            
-            
         # do the dirs - set the root if there is one (grumble)
         for option in ['cachedir', 'logfile']:
             path = self.configdata[option]
@@ -275,29 +325,38 @@ class yumconf(object):
             setattr(self, option, rootedpath)
         
         # bandwidth limit options require special parsing
-        for option, getfunc in (('bandwidth', self.cfg.getbytes), 
-                                ('throttle', self.cfg.getthrottle)):
+        for option, getfunc in [('bandwidth', self.cfg.getbytes), 
+                                ('throttle', self.cfg.getthrottle)]:
             value = getfunc('main', option, 0)
+            value = variableReplace(self.yumvar, value)
             self.configdata[option] = value
             setattr(self, option, value)
 
-        
-        
-        # get our variables parsed
-        self.yumvar = self._getEnvVar()
-        self.yumvar['basearch'] = rpmUtils.arch.getBaseArch() # FIXME make this configurable??
-        self.yumvar['arch'] = rpmUtils.arch.getCanonArch() # FIXME make this configurable??
-        # figure out what the releasever really is from the distroverpkg
-        self.yumvar['releasever'] = self._getsysver()
+        # http_caching needs special handling
+        value = self.cfg.getselection('main', 'http_caching', 'all', 
+                HTTP_CACHING_VALS)
+        setattr(self, 'http_caching', value)
+        self.configdata['http_caching'] = value
         
         # weird ones
-        for option in ['commands', 'installonlypkgs', 'kernelpkgnames', 'exclude']:
-            self.configdata[option] = variableReplace(self.yumvar, self.configdata[option])
-            self.configdata[option] = variableReplace(self.yumvar, self.configdata[option])
+        for option in ['commands', 'installonlypkgs', 'kernelpkgnames',
+            'exclude', 'reposdir', 'exactarchlist', 'pluginpath']:
+            self.configdata[option] = variableReplace(self.yumvar, 
+                    self.configdata[option])
+            setattr(self, option, self.configdata[option])
 
         # make our lists into lists. :)
-        for option in ['exclude', 'installonlypkgs', 'kernelpkgnames', 'tsflags']:
+        for option in ['exclude', 'installonlypkgs', 'kernelpkgnames', 
+                'tsflags', 'reposdir', 'exactarchlist', 'pluginpath']:
             self.configdata[option] = parseList(self.configdata[option])
+            setattr(self, option, self.configdata[option])
+
+        # Check that plugin paths are all absolute
+        for path in self.pluginpath:
+            if path[:1] != '/':
+                raise Errors.ConfigError(
+                        "All plugin search paths must be absolute")
+
 
     def listConfigOptions(self):
         """return list of options available for global config"""
@@ -382,51 +441,64 @@ def cfgParserRepo(section, yumconfig, cfgparser):
        Returns a repos.Repository object
        """
     
-    
     thisrepo = Repository(section)
+    thisrepo.cfgparser = cfgparser
     thisrepo.set('yumvar', yumconfig.yumvar)
+
+    for keyword in ['proxy_username', 'proxy', 'proxy_password',
+                    'retries', 'failovermethod', 'name']:
+        val = cfgparser._getoption(section, keyword, yumconfig.getConfigOption(keyword))
+        val = variableReplace(yumconfig.yumvar, val)
+        thisrepo.set(keyword, val)
     
-    enabled = cfgparser._getboolean(section, 'enabled', 1)
-    name = cfgparser._getoption(section, 'name', section)
-    name = variableReplace(yumconfig.yumvar, name)
-    thisrepo.set('name', name) 
-    thisrepo.set('enabled', enabled)
+    for keyword in ['gpgcheck', 'keepalive', 'enablegroups', 'enabled']:
+        val = cfgparser._getoption(section, keyword, yumconfig.getConfigOption(keyword))
+        val = variableReplace(yumconfig.yumvar, val)
+        if type(val) not in (True, False):
+            val = str(val)
+            if val.lower() not in BOOLEAN_STATES:
+                raise Errors.RepoError, 'Invalid value in repo config for %s::%s' % (section, keyword)
+            val = BOOLEAN_STATES[val.lower()]
+        thisrepo.set(keyword, val)
     
+    for (keyword, getfunc) in [('bandwidth', cfgparser.getbytes),
+                               ('throttle', cfgparser.getthrottle),
+                               ('timeout', cfgparser._getfloat)]:
+        val = getfunc(section, keyword, yumconfig.getConfigOption(keyword))
+        thisrepo.set(keyword, val)
+
+
+    # lists and weird items
     baseurl = cfgparser._getoption(section, 'baseurl', [])
     baseurls = parseList(baseurl)
     mirrorlistfn = cfgparser._getoption(section, 'mirrorlist', None)
     mirrorlistfn = variableReplace(yumconfig.yumvar, mirrorlistfn)
     thisrepo.set('mirrorlistfn', mirrorlistfn)
     thisrepo.set('baseurls', baseurls)
-    
 
-    gpgkey = cfgparser._getoption(section, 'gpgkey', '')
-    if gpgkey:
+    thisrepo.set('http_caching', cfgparser.getselection(
+            section, 'http_caching', 
+            yumconfig.getConfigOption('http_caching'),
+            HTTP_CACHING_VALS
+            )
+        )
+
+    # Parse and check gpgkey URLs
+    gpgkeys = cfgparser._getoption(section, 'gpgkey', '')
+    gpgkeys = variableReplace(yumconfig.yumvar, gpgkeys)
+    gpgkeys = parseList(gpgkeys)
+    for gpgkey in gpgkeys:
         (s,b,p,q,f,o) = urlparse.urlparse(gpgkey)
         if s not in ('http', 'ftp', 'file', 'https'):
-            print 'gpgkey must be ftp, http[s], or file URL, ignoring - %s' % gpgkey
-            gpgkey = ''
-    thisrepo.set('gpgkey', gpgkey)
+            raise Errors.ConfigError, 'gpgkey must be ftp, http[s], or file URL: %s' % gpgkey
+    thisrepo.set('gpgkey', gpgkeys)
 
-
-    for keyword in ['proxy_username', 'proxy', 'proxy_password', 
-                    'retries', 'failovermethod']:
-
-        thisrepo.set(keyword, cfgparser._getoption(section, keyword, \
-                     yumconfig.getConfigOption(keyword)))
-                     
-    for keyword, getfunc in (('bandwidth', cfgparser.getbytes),
-                             ('throttle', cfgparser.getthrottle)):
-        thisrepo.set(keyword, getfunc(section, keyword, 
-                    yumconfig.getConfigOption(keyword)))
-
-    for keyword in ['gpgcheck', 'keepalive']:
-        thisrepo.set(keyword, cfgparser._getboolean(section, \
-                     keyword, yumconfig.getConfigOption(keyword)))
-    
-    for keyword in ['timeout']:
-        thisrepo.set(keyword, cfgparser._getfloat(section, \
-                     keyword, yumconfig.getConfigOption(keyword)))
+    # check out the proxy url
+    if thisrepo.proxy:
+        (s,b,p,q,f,o) = urlparse.urlparse(thisrepo.proxy)
+        if s not in ('http', 'ftp', 'https'):
+            raise Errors.ConfigError, 'proxy must be ftp or http[s] URL: %s' % thisrepo.proxy
+            thisrepo.set('proxy', None)
     
     excludelist = cfgparser._getoption(section, 'exclude', [])
     excludelist = variableReplace(yumconfig.yumvar, excludelist)
@@ -438,14 +510,7 @@ def cfgParserRepo(section, yumconfig, cfgparser):
     includelist = parseList(includelist)
     thisrepo.set('includepkgs', includelist)
 
-    thisrepo.set('enablegroups', cfgparser._getboolean(section, 'enablegroups', 1))
-    
-    cachedir = os.path.join(yumconfig.getConfigOption('cachedir'), section)
-    pkgdir = os.path.join(cachedir, 'packages')
-    hdrdir = os.path.join(cachedir, 'headers')
-    thisrepo.set('cachedir', cachedir)
-    thisrepo.set('pkgdir', pkgdir)
-    thisrepo.set('hdrdir', hdrdir)
+    thisrepo.set('basecachedir', yumconfig.getConfigOption('cachedir'))
     
     return thisrepo
 
@@ -466,8 +531,7 @@ def parseList(value):
     (value, count) = commarepl.subn(' ', value)
     listvalue = value.split()
     return listvalue
-        
-
+      
 
 class confpp:
     """
@@ -647,6 +711,7 @@ class confpp:
     def geturl(self): return self.name
 
 def main(args):
+
     myfile = args[0]
     if len(args) > 1:
         if args[1] == '--dump':
@@ -681,32 +746,36 @@ def main(args):
     # does not read recursively
     # read each of them in using confpp, then parse them same as any other repo
     # section - as above.
-    reposdir = conf.reposdir
-    if os.path.exists(conf.installroot + '/' + reposdir):
-        reposdir = conf.installroot + '/' + reposdir
-    
-    reposglob = reposdir + '/*.repo'
-    if os.path.exists(reposdir) and os.path.isdir(reposdir):
-        repofn = glob.glob(reposglob)
-        repofn.sort()
-        
-        for fn in repofn:
-            if not os.path.isfile(fn):
-                continue
+
+    reposdirs = []
+    for dir in conf.reposdir:
+        if os.path.exists(conf.installroot + '/' + dir):
+            reposdirs.append(conf.installroot + '/' + dir)
+
+    repofn = []
+    for reposdir in reposdirs:
+        if os.path.exists(reposdir) and os.path.isdir(reposdir):
+            reposglob = reposdir + '/*.repo'
+            repofn.extend(glob.glob(reposglob))
+
+    repofn.sort()
+    for fn in repofn:
+        if not os.path.isfile(fn):
+            continue
+        try:
+            cfg, sections = parseDotRepo(fn)
+        except Errors.ConfigError, e:
+            print >> sys.stderr, e
+            continue
+
+        for section in sections:
             try:
-                cfg, sections = parseDotRepo(fn)
-            except Errors.ConfigError, e:
+                thisrepo = cfgParserRepo(section, conf, cfg)
+            except (Errors.RepoError, Errors.ConfigError), e:
                 print >> sys.stderr, e
                 continue
-
-            for section in sections:
-                try:
-                    thisrepo = cfgParserRepo(section, conf, cfg)
-                except (Errors.RepoError, Errors.ConfigError), e:
-                    print >> sys.stderr, e
-                    continue
-                else:
-                    reposlist.append(thisrepo)
+            else:
+                reposlist.append(thisrepo)
 
     # got our list of repo objects
     reposlist.sort()
