@@ -17,6 +17,7 @@
 import os
 import os.path
 import re
+import types
 
 import rpmUtils.transaction
 import rpmUtils.miscutils
@@ -41,7 +42,8 @@ class Depsolve:
         ts_flags_to_rpm = { 'noscripts': rpm.RPMTRANS_FLAG_NOSCRIPTS,
                             'notriggers': rpm.RPMTRANS_FLAG_NOTRIGGERS,
                             'nodocs': rpm.RPMTRANS_FLAG_NODOCS,
-                            'test': rpm.RPMTRANS_FLAG_TEST}
+                            'test': rpm.RPMTRANS_FLAG_TEST,
+                            'repackage': rpm.RPMTRANS_FLAG_REPACKAGE}
         
         self.ts.setFlags(0) # reset everything.
         
@@ -74,12 +76,17 @@ class Depsolve:
         if flags == 0:
             flags = None
         
-        (r_e, r_v, r_r) = rpmUtils.miscutils.stringToVersion(version)
+
+        if type(version) in (types.StringType, types.NoneType):
+            (r_e, r_v, r_r) = rpmUtils.miscutils.stringToVersion(version)
+        elif type(version) in (types.TupleType, types.ListType): # would this ever be a ListType?
+            (r_e, r_v, r_r) = version
+        
         defSack = ListPackageSack() # holder for items definitely providing this dep
         
         for po in pkgs:
             self.log(5, 'Potential match for %s from %s' % (name, po))
-            if name[0] == '/' and version is None:
+            if name[0] == '/' and r_v is None:
                 # file dep add all matches to the defSack
                 defSack.addPackage(po)
                 continue
@@ -99,14 +106,43 @@ class Depsolve:
             return 1
         
         provides = po.getProvidesNames()
-        if 'kernel-modules' in provides:
-            return 1
-        
-        if 'kernel' in provides:
+        if filter (lambda prov: prov in self.conf.installonlypkgs, provides):
             return 1
         
         return 0
-        
+
+    def handleKernelModule(self, txmbr):
+        """Figure out what special magic needs to be done to install/upgrade
+           this kernel module."""
+
+        def getKernelReqs(hdr):
+            kernels = ["kernel-%s" % a for a in rpmUtils.arch.arches.keys()]
+            reqs = []
+            names = hdr[rpm.RPMTAG_REQUIRENAME]
+            flags = hdr[rpm.RPMTAG_REQUIREFLAGS]
+            ver =   hdr[rpm.RPMTAG_REQUIREVERSION]
+            if names is not None:
+                reqs = zip(names, flags, ver)
+            return filter(lambda r: r[0] in kernels, reqs)
+
+        kernelReqs = getKernelReqs(txmbr.po.returnLocalHeader())
+        instPkgs = self.rpmdb.returnTupleByKeyword(name=txmbr.po.name)
+        for pkg in instPkgs:
+            hdr = self.rpmdb.returnHeaderByTuple(pkg)[0]
+            instKernelReqs = getKernelReqs(hdr)
+            
+            for r in kernelReqs:
+                if r in instKernelReqs:
+                    # we know that an incoming kernel module requires the
+                    # same kernel as an already installed module of the
+                    # same name.  "Upgrade" this module instead of install
+                    po = packages.YumInstalledPackage(hdr)
+                    self.tsInfo.addErase(po)
+                    self.log(4, 'Removing kernel module %s upgraded to %s' %
+                             (po, txmbr.po))
+                    break
+       
+
     def populateTs(self, test=0, keepold=1):
         """take transactionData class and populate transaction set"""
 
@@ -135,6 +171,8 @@ class Depsolve:
                 rpmfile = txmbr.po.localPkg()
                 
                 if txmbr.ts_state == 'u':
+                    if txmbr.po.name.startswith("kernel-module-"):
+                        self.handleKernelModule(txmbr)
                     if self.allowedMultipleInstalls(txmbr.po):
                         self.log(5, '%s converted to install' % (txmbr.po))
                         txmbr.ts_state = 'i'
@@ -350,12 +388,14 @@ class Depsolve:
     def _requiringFromInstalled(self, requiringPkg, requirement, errorlist):
         """processes the dependency resolution for a dep where the requiring 
            package is installed"""
+           
         (name, arch, epoch, ver, rel) = requiringPkg
+        requiringPo = self.getInstalledPackageObject(requiringPkg)
+        
         (needname, needflags, needversion) = requirement
         niceformatneed = rpmUtils.miscutils.formatRequire(needname, needversion, needflags)
         checkdeps = 0
         missingdep = 0
-        reqpkg_print = '%s.%s %s:%s-%s' % requiringPkg
         
         # we must first find out why the requirement is no longer there
         # we must find out what provides/provided it from the rpmdb (if anything)
@@ -387,11 +427,11 @@ class Depsolve:
             thismode = self.tsInfo.getMode(name=i_n, arch=i_a, 
                             epoch=i_e, ver=i_v, rel=i_r)
                         
-            if thismode is None and self.conf.exactarch:
+            if thismode is None and i_n in self.conf.exactarchlist:
                 # check for mode by the same name+arch
                 thismode = self.tsInfo.getMode(name=i_n, arch=i_a)
             
-            if thismode is None and not self.conf.exactarch:
+            if thismode is None and i_n not in self.conf.exactarchlist:
                 # check for mode by just the name
                 thismode = self.tsInfo.getMode(name=i_n)
             
@@ -407,20 +447,43 @@ class Depsolve:
         
         if needmode in ['e']:
                 self.log(5, 'TSINFO: %s package requiring %s marked as erase' %
-                                (reqpkg_print, needname))
-                pkg = packages.YumInstalledPackage(self.rpmdb.returnHeaderByTuple(requiringPkg)[0])
-                txmbr = self.tsInfo.addErase(pkg)
+                                (requiringPo, needname))
+                txmbr = self.tsInfo.addErase(requiringPo)
                 txmbr.setAsDep(pkgtup = needpkgtup)
                 checkdeps = 1
         
         if needmode in ['i', 'u']:
             self.doUpdateSetup()
+            obslist = []
+            # check obsoletes first
+            if self.conf.obsoletes:
+                if self.up.obsoleted_dict.has_key(requiringPo.pkgtup):
+                    obslist = self.up.obsoleted_dict[requiringPo.pkgtup]
+                    self.log(4, 'Looking for Obsoletes for %s' % requiringPo)
+                
+            if len(obslist) > 0:
+                po = None
+                for pkgtup in obslist:
+                    po = self.getPackageObject(pkgtup)
+                if po:
+                    for (new, old) in self.up.getObsoletesTuples():
+                        if po.pkgtup == new:
+                            txmbr = self.tsInfo.addObsoleting(po, requiringPo)
+                            self.tsInfo.addObsoleted(requiringPo, po)
+                            txmbr.setAsDep(pkgtup = needpkgtup)
+                            self.log(5, 'TSINFO: Obsoleting %s with %s to resolve dep.' % (requiringPo, po))
+                            checkdeps = 1
+                            return checkdeps, missingdep 
+                
+            
+            # check updates second
+            uplist = []                
             uplist = self.up.getUpdatesList(name=name)
+            # if there's an update for the reqpkg, then update it
             
             po = None
-            # if there's an update for the reqpkg, then update it
             if len(uplist) > 0:
-                if not self.conf.exactarch:
+                if name not in self.conf.exactarchlist:
                     pkgs = self.pkgSack.returnNewestByName(name)
                     archs = []
                     for pkg in pkgs:
@@ -436,8 +499,7 @@ class Depsolve:
             if po:
                 for (new, old) in self.up.getUpdatesTuples():
                     if po.pkgtup == new:
-                        updated_pkg = packages.YumInstalledPackage(self.rpmdb.returnHeaderByTuple(old)[0])
-                        txmbr = self.tsInfo.addUpdate(po, updated_pkg)
+                        txmbr = self.tsInfo.addUpdate(po, requiringPo)
                         txmbr.setAsDep(pkgtup = needpkgtup)
                         self.log(5, 'TSINFO: Updating %s to resolve dep.' % po)
                 checkdeps = 1
@@ -627,7 +689,7 @@ class Depsolve:
 
             # if there's an update for the reqpkg, then update it
             if len(uplist) > 0:
-                if not self.conf.exactarch:
+                if confname not in self.conf.exactarchlist:
                     pkgs = self.pkgSack.returnNewestByName(confname)
                     archs = []
                     for pkg in pkgs:
